<?php
/*
Plugin Name: Get URL Cron
Plugin URI: https://json-content-importer.com/geturlcron
Description: Request http-URLs via Cronjobs and check the result. Retry if needed. Monitor and Check Websites.
Version: 1.2
Author: Bernhard Kux
Author URI: http://www.kux.de/
License: GPLv3
License URI: http://www.gnu.org/licenses/gpl-3.0.html
*/

/* block direct requests */

defined('ABSPATH') OR exit;
if ( !function_exists( 'add_action' )) {
	echo 'Hello, this is a plugin: You must not call me directly.';
	exit;
}
define( 'GETURLCRON_VERSION', '1.2' );  // current version number


class GetUrlCron {	
	private $urlSettingsArr = array();
	private static $nooffields = 15;
	private static $logfile = "";
	private $subaction = "";
	private $action = "";
	public static $fi = array(
			"url" => 1,
			"interval" => 2,
			"startdate" => 3,
			"retries" => 4,
			"requiredformat" => 5,
			"requiredjsonfield" => 6,
			"sendmail" => 8,
			);

	public static $fi_size = array(
			"url" => 100,
			"interval" => 20,
			"startdate" => 25,
			"retries" => 2,
			"requiredformat" => 5,
			"requiredjsonfield" => 10,
			"sendmail" => 1,
			);
			
	public static $reqformatArr = array(
			"any" => "string",
			"json" => "json",
		);



	protected function __construct() {
		#srequire_once plugin_dir_path( __FILE__ ) . '/options.php';

		add_action('admin_menu', array( $this, 'geturlcron_menu'));

		$this->geturlcron_setlogfile();

		add_filter( 'cron_schedules',  array( $this, 'geturlcron_recurrence_interval' ) );
		add_action( 'init', array( $this, 'geturlcron_action_handle' ) );

		register_activation_hook(__FILE__, array($this, 'geturlcron_activatejobs'));
		register_deactivation_hook(__FILE__, array($this, 'geturlcron_unschedulejobs'));
		$this->geturlcron_set_urlSettingsarr();

		for ($no = 1; $no <= count($this->urlSettingsArr); $no++) {
			add_action('geturlcron_event-'.$no, array( $this, 'geturlcron_add_action_cronjob' ) );
		}
	}
	
	
public function geturlcron_detailsettings_page() {
	echo "<h1>Basic Settings</h1>";
	echo '<form method="post" action="admin.php?page=geturlcrondetailsettingslug">';
	submit_button();
	echo '<table class="widefat striped">';
	echo '<input type="hidden" name="subaction" value="settings">';
	settings_fields( 'geturlcron-options-details' ); 
	do_settings_sections( 'geturlcron-options-details' ); 

	echo "<tr><td>";
	echo "<h2>E-Mailadress for Statusmessages: separate multiple by space or , or ;</h2>";
	$mailadr = trim(get_option('geturlcron-emailadr'));
	echo $this->geturlcron_check_mailadress_list($mailadr);
	echo '<input type=text size=200 name=geturlcron-emailadr value="'.$mailadr.'">';
	echo "</td></tr>";
	echo "<tr><td>";
	echo "<h2>Set timeout</h2>";
	echo "Set the timeout for the http-requests (default 60 sec):<br>";
	$timeout = trim(get_option('geturlcron-timeout'));
	if (!($timeout>0)) {
		$timeout = "60";
	}
	echo '<input type=text size=5 name=geturlcron-timeout value="'.$timeout.'">';
	echo "</td></tr>";
	echo "<tr><td>";
	echo "<h2>Max. age of logentries</h2>";

    echo "Delete Logfile-Entires older than days:<ul><li>-1 : delete logfile and do not log</li><li>0 : do not delete anything</li><li>any number : max. age of the logfile-entries, default is 20 days</li></ul>";

	$deldays = trim(get_option('geturlcron-dellog-days'));
	echo '<input type=text size=5 name=geturlcron-dellog-days value="'.$deldays.'">';
	echo "</td></tr>";
	echo "<tr><td>";
	
	echo "<h2>Complete delete when uninstalling?</h2>";
    echo "On default, not all data of this plugin is deleted:";
    echo "Only if the following checkbox is activated, also templates and the above option-data are deleted<br>";
	$checkeddelall = "";
	if (get_option('geturlcron-uninstall-deleteall') == 1) {
		$checkeddelall = "checked=checked";
	}
	
    echo '<input type="checkbox" name="geturlcron-uninstall-deleteall" value="1" '.$checkeddelall.' /> delete all, incl. logfiles';

	echo "</td></tr>";
	echo "<tr><td>";

	echo "<h2>Example</h2>";
    echo "For trying the plugin you might use a URL like this one:<br>";
	$exampleurl = "http://worldtimeapi.org/api/timezone/Europe/Berlin";
	echo '<a href="'.$exampleurl.'" target="_blank">'.$exampleurl.'</a><br>';	
	echo "<ul>";
	echo "<li>1. Select JSON as requiredformat and 'timezone' as requiredjsonfield</li>";
	echo "<li>2. Save Settings</li>";
	echo "<li>3. Then executing the CronJob by clicking 'execute job'</li>";
	echo "<li>4. Switching to 'Show Logs' should show you the results</li></ul>";
	
	echo "</td></tr>";
	echo "</table>";
	submit_button();
	echo "</form>";
}

public function geturlcron_menu() {
	add_menu_page('Get URL Cron', 'Get URL Cron', 'administrator', 'unique_geturlcron_menu_slug', array($this, 'geturlcron_settings_page'), 'dashicons-clock');
	add_submenu_page('unique_geturlcron_menu_slug', 'Set CronJobs', 'Set CronJobs', 'administrator', 'geturlcronsettingspage', array($this, 'geturlcron_settings_page'));
	add_submenu_page('unique_geturlcron_menu_slug', 'Show CronJobs', 'Show CronJobs', 'administrator', 'geturlcronjobslistdslug', array($this, 'geturlcron_cronjobs_page'));
	add_submenu_page('unique_geturlcron_menu_slug', 'Show Logs', 'Show Logs', 'administrator', 'geturlcronlogslug', array($this, 'geturlcron_logs_page'));
	add_submenu_page('unique_geturlcron_menu_slug', 'Basic Settings', 'Basic Settings', 'administrator', 'geturlcrondetailsettingslug', array($this, 'geturlcron_detailsettings_page'));
	remove_submenu_page('unique_geturlcron_menu_slug', 'unique_geturlcron_menu_slug');
	add_action( 'admin_init', array($this, 'register_geturlcronsettings' ));
}	


public function geturlcron_logs_page() {
	$logfile = $this->geturlcron_getlogfile();
	$deldays = trim(get_option('geturlcron-dellog-days'));
	if ($deldays==-1) {
		@unlink($logfile);
		echo '<h1>Logfile deleted!</h1>See settings and check "Delete Logfile-Entires older than": "-1" means delete logfile';
		return TRUE;
	}
	echo "<h1>Logs</h1>";
	
	if (!file_exists($logfile)) {
		echo "emtpy logfile up to now...";
		return TRUE;
	}
	$logf = file_get_contents($logfile);
	$separator = " /// ";
	$separator2 = "=";
	
	#echo $logf; return true;
	$logfArr1 = explode("\n",$logf);
	
	#echo count($logfArr1); return true;
	$outhead = '<table class="widefat" border=1>';
		$outhead .= "<tr bgcolor=yellow>";
		$outhead .= "<td>";
		$outhead .= "id of run";
		$outhead .= "</td><td>";
		$outhead .= "status";
		$outhead .= "</td><td>";
		$outhead .= "log entry";
		$outhead .= "</td><td>";
		$outhead .= "retires";
		$outhead .= "</td><td>";
		$outhead .= "json status";
		$outhead .= "</td><td>";
		$outhead .= "runtime (sec)";
		$outhead .= "</td><td>";
		$outhead .= "url";
		$outhead .= "</td><td>";
		$outhead .= "response";
		$outhead .= "</td>";
		$outhead .= "</tr>";
		$deldays = trim(get_option('geturlcron-dellog-days'));
		$count = 0 ;
		$logfArr11 = array();
		$statuscheck = array();
	for ($r = count($logfArr1); $r >=0; $r--) {
		if (empty($logfArr1[$r])) {
			continue;
		}
		$logfArr2 = explode($separator , $logfArr1[$r]);
		#echo "<hr>$r  -- ".$logfArr1[$r]."<hr>";
		$id = $logfArr2[0];
		$status = $logfArr2[8];
		$statkey = trim($id)."-".trim($status);
		@$statuscheck{$statkey}++;
		#echo $statkey."-".strlen($statkey)."<br>";
		$tArr = explode("time=", $logfArr1[$r]);
		$tArr1 = explode(" ", $tArr[1]);
		$deleteentry = FALSE;
		if ($deldays>0) {
			if ($tArr1[0]<10) {
				$deleteentry = TRUE;
			} else {
				$ageofentryindays = (time()-$tArr1[0])/86400;
				if ($ageofentryindays>$deldays) {
					$deleteentry = TRUE;
				}
			}
			if ($deleteentry) {
				#echo "DEL: ".$ageofentryindays." - $deldays <hr>";
			} else {
				#echo "OK:  ".$ageofentryindays." - $deldays <hr>";
				$logfArr11[$count] = $logfArr1[$r];
				$count++;
			}
		} else {
			$logfArr11[$count] = $logfArr1[$r];
			$count++;
		}
	}
	if ($deleteentry) {
		$newlogfile = join("\n", $logfArr11);
		$fsc = file_put_contents($logfile, $newlogfile."\n");
	}
	
	echo $outhead;
	for ($r = count($logfArr11); $r >=0; $r--) {
		if (empty($logfArr1[$r])) {
			continue;
		}
		$logfArr2 = explode($separator, $logfArr1[$r]);
		if (empty($logfArr2[1])) {
			continue;
		}
		$lga3 = explode($separator2, $logfArr2[1],2);
		if (($lga3[1]==0) || count($logfArr2)==0) {
			continue;
		}
		$id = trim($logfArr2[0]);
		$status = trim($logfArr2[8]);
		$bgcol = "#ffffff"; 
		if ($status=="try" && @$statuscheck{$id."-fail"}>0) {
			$bgcol = "#ffa099"; 
		}
		#echo $id."-OK"." : ".strlen($id."-OK")." : ".$statuscheck{$id."-OK"}."<br>";
		if ($status=="try" && $statuscheck{$id."-OK"}>0) {
			$bgcol = "#9edeaa"; 
		}
		if ($status=="OK" && $statuscheck{$id."-try"}>0) {
			$bgcol = "#9edeaa"; 
		}
		if ($status=="fail") {
			$bgcol = "#ffa099"; 
		}
		if ($status=="schedule") {
			$bgcol = "#ffbb00"; 
		}

		echo "<tr bgcolor=$bgcol>";
		echo "<td>";
		echo $id;
		echo "</td><td>";
		echo $status;
		echo "</td><td>";
		echo date("Y-m-d, H:i:s", $lga3[1]);
		echo "</td><td>";
		$lga43 = explode($separator2, $logfArr2[3],2);
		echo $lga43[1];
		echo "</td><td>";
		echo $logfArr2[4];
		echo "</td><td>";
		$lga43 = explode($separator2, $logfArr2[6],2);
		echo $lga43[1];
		echo "</td><td>";
		$lga43 = explode($separator2, $logfArr2[5],2);
		echo $lga43[1];
		echo "</td><td>";
		$lga43 = explode($separator2, $logfArr2[7],2);
		$lga43[1] = chunk_split($lga43[1], 200, ' '); # insert blank every 200char for linebreaks
		echo esc_html($lga43[1]);
		echo "</td>";
		echo "</tr>";
	}
	echo "</table>";
	return TRUE;
}

public function geturlcron_cronjobs_page() {
	#$cjArr = wp_get_schedules(); #gives only few info
	$cjArr = _get_cron_array();

	$out = "";
	$outelse = "";
	$plugincronjobs = 0;
	$nonplugincronjobs = 0;
	foreach($cjArr as $k => $v) {
		foreach($v as $k1 => $v1) {
			$noofjob = preg_replace("/geturlcron_event-/", "", $k1);
			$showcronjob = TRUE;
			$op = get_option("geturlcron-url-".$noofjob);
			if ($this->is_relative_url($op)) {
				# relative path
				$op = $this->add_domain_to_url($op);
			}
			if (empty($op)) {
				$jobhook = "geturlcron_event-".$noofjob;
				wp_unschedule_event( "", $jobhook );
				wp_clear_scheduled_hook( $jobhook );
				$showcronjob = FALSE;
			}
			if ($noofjob>=1 && $showcronjob) {
				foreach($v1 as $k2 => $v2) {
					$intv = $v2["schedule"];
					if (empty($intv)) {
						$intv = "run only once";
					}
				}
				$plugincronjobs++;
				$out .= "<tr>";
				$out .= "<td>";
				$out .= $k1;
				$out .= "</td>";
				$out .= "<td>";
				$opout = trim(stripslashes($op));
				if (preg_match("/^\[/",$opout)) {
					$out .= "execute Shortcode $opout";
				} else {
					$out .= "<a href=$op target=_blank>".$opout."</a>";
				}
				$out .= "</td>";
				$out .= "<td>";
				$out .= $intv;
				$out .= "</td>";
				$out .= "<td>";
				$std = get_option("geturlcron-startdate-".$noofjob);
				$out .= "$std</td>";
				$out .= "<td>";
				$nexttime = wp_next_scheduled($k1, array());
				$nextdate = "";
				$nextdist = "";
				if ($nexttime>0) {
					$nextdate = date("Y-m-d, H:i:s", $nexttime);
					$nextdistVal = $nexttime-time();
					$nextdist = date("H:i:s", $nextdistVal);					
				}
				$out .= "$nextdate</td>";
				$out .= "<td>";
				$out .= "$nextdist</td>";
				$out .= "</tr>";
			} else {
				# other cronjobs
				$nonplugincronjobs++;
				$outelse .= "<tr>";
				$outelse .= "<td>";
				$outelse .= $k1;
				$outelse .= "</td>";
				$outelse .= "<td>";
				$nexttime = wp_next_scheduled($k1, array());
				$nextdate = "";
				$nextdist = "";
				if ($nexttime>0) {
					$nextdate = date("Y-m-d, H:i:s", $nexttime);
					$nextdistVal = $nexttime-time();
					$nextdist = date("H:i:s", $nextdistVal);					
				}
				$outelse .= $nextdate;
				$outelse .= "</td>";
				$outelse .= "<td>";
				$outelse .= $nextdist;
				$outelse .= "</td>";
				$outelse .= "</tr>";
			}
		}
	}
	$out .= "</table>";
	$outelse .= "</table>";
	
	$outhead = '<h1>'.$plugincronjobs.' Cron-Jobs defined by this Plugin:</h1><table class="widefat striped">';
	$outhead .= "<tr><td bgcolor=yellow>Cronjob</td><td bgcolor=yellow>URL</td><td bgcolor=yellow>interval</td><td bgcolor=yellow>first run</td><td bgcolor=yellow>next run</td><td bgcolor=yellow>distance to next run</td></tr>";
	$outheadelse = '<h1>'.$nonplugincronjobs.' other Cron-Jobs:</h1><table class="widefat striped">';
	$outheadelse .= "<tr><td bgcolor=yellow>Cronjob</td><td bgcolor=yellow>next run</td><td bgcolor=yellow>distance to next run</td></tr>";
	echo $outhead.$out;
	echo $outheadelse.$outelse;
	echo "<hr><h2>Current time on this server: ".date("Y-m-d, H:i:s")."</h2>";
	echo "All times like time and distance of the next run are calculated with this time!";
}


public function geturlcron_settings_page() {
	echo '<h1>Get URL Cron Jobs:</h1>';
	echo '<form method="post" action="admin.php?page=geturlcronsettingspage">';
    submit_button();
		echo '<input type="hidden" name="subaction" value="savecronjobs">';
		settings_fields( 'geturlcron-options' ); 
		do_settings_sections( 'geturlcron-options' ); 
		$fi = self::$fi;
		$fi_size = self::$fi_size;
		$reqformatArr = self::$reqformatArr;
		
		echo '<table class="widefat striped">';
		echo "<tr>";
		echo "<td bgcolor=yellow>";
		echo "No";
		echo "</td>";
		foreach($fi as $k => $v) {
			echo "<td bgcolor=yellow>";
			if ($k=="url") {
				echo "URL or WP-Shortcode (if URL starts with \"/\" the Sitedomain is added in the Cronjob-URL)";
			} else {
				echo $k;
			}
			echo "</td>";
		}
		echo "<td bgcolor=yellow>";
		echo "execute job";
		echo "</td>";
		echo "</tr>";
		
		$nooffields = $this->geturlcron_getnooffields();
		
		for ($r = 1; $r <= $nooffields; $r++) {
			echo "<tr>";
			echo "<td>";
			echo $r;
			echo "</td>";
			foreach($fi as $k => $v) {
				echo "<td>";
				$ki = "geturlcron-".$k."-".$r;
				$op = get_option($ki);
				if ($k=="interval") { 
					$cronschedulesArr = wp_get_schedules();
					#print_r($cronschedulesArr);exit;

					if ($op=="") { $op = "daily"; }
					echo "<select name=$ki>";
					$scArr_display = array();
					$scArr_key = array();
					foreach($cronschedulesArr as $csk => $csv) {
						$scArr_display{$csv["interval"]} = $csv["display"];
						$scArr_key{$csv["interval"]} = $csk;
					}
					ksort($scArr_key, SORT_NUMERIC);


					foreach($scArr_key as $csk => $csv) {
						$csel = "";
						if ($op==$csv) {
							$csel = " selected ";
						}
						echo "<option value=".$csv." $csel>".$scArr_display[$csk];
					}
					echo "</select>";
				} else if ($k=="requiredformat") {
					echo "<select name=$ki>";
					foreach($reqformatArr as $csk => $csv) {
						$csel = "";
						if ($op==$csk) {
							$csel = " selected ";
						}
						echo "<option value=".$csk." $csel>$csv ";
					}
					echo "</select>";
				} else if ($k=="retries") {
					echo "<select name=$ki>";
					for ($rr = 1; $rr <= 10; $rr++) {
						$csel = "";
						if ($op==$rr) {
							$csel = " selected ";
						}
						echo "<option value=".$rr." $csel>$rr ";
					}
					echo "</select>";
				} else if ($k=="sendmail") {
					$sel = "";
					if ($op=="yes" || (!isset($op))) {
						$csel = " checked ";
					}
					echo "<input type=checkbox $csel name=$ki value=yes \>";
				} else {
					$placeholder = "";
					if ($k=="startdate") {
						$placeholder = "YYYY-MM-DD hh:mm:ss";
					}
					if ($k=="url") {
					$placeholder = "http... OR /path... OR [shortcode id...]";
					}
					$opout = stripslashes($op);
					echo '<input type=text placeholder="'.$placeholder.'" name="'.$ki.'" value="'.esc_html($opout).'" size='.$fi_size[$k].'>';
					#echo '<input type=text placeholder="'.$placeholder.'" name="'.$ki.'" value="'.$op.'" size='.$fi_size[$k].'>';
				}
				echo "</td>";
			}
			echo "<td>";
			$nonce = wp_create_nonce( 'getcronurl' );
			$url = "?page=unique_geturlcron_menu_slug&action=geturlcron&no=$r&hash=$nonce";
			echo "<a href=$url>execute job</a>";
			echo "</td>";
			echo "</tr>";
		}
		echo "</table>";
		submit_button(); 
		echo "</form>";
}
	
public function register_geturlcronsettings() {
	register_setting( 'geturlcron-options-details', 'geturlcron-emailadr' );
	register_setting( 'geturlcron-options-details', 'geturlcron-timeout' );
	register_setting( 'geturlcron-options-details', 'geturlcron-uninstall-deleteall' );
	register_setting( 'geturlcron-options-details', 'geturlcron-dellog-days' );
	$nooffields = $this->geturlcron_getnooffields();
	for ($r = 1; $r <= $nooffields; $r++) {
		register_setting( 'geturlcron-options', 'geturlcron-url-'.$r );
		register_setting( 'geturlcron-options', 'geturlcron-interval-'.$r );
		register_setting( 'geturlcron-options', 'geturlcron-startdate-'.$r );
		register_setting( 'geturlcron-options', 'geturlcron-retries-'.$r );
		register_setting( 'geturlcron-options', 'geturlcron-requiredjsonfield-'.$r );
		register_setting( 'geturlcron-options', 'geturlcron-requiredformat-'.$r );
		register_setting( 'geturlcron-options', 'geturlcron-sendmail-'.$r );
	}
}	

	private function is_relative_url($url) {
		if (preg_match("/^\//", $url)) {
			return TRUE;
		}
		return FALSE;
	}
	
	private function add_domain_to_url($url) {
		return home_url().$url;
	}
	
	private function geturlcron_set_urlSettingsarr() {
		for ($r = 1; $r <= self::$nooffields; $r++) {
			foreach(self::$fi as $k => $v) {
				$ki = "geturlcron-".$k."-".$r;
				$op = get_option($ki);
				if ($k=="url" && $this->is_relative_url($op)) {
					$op = $this->add_domain_to_url($op);
				}
				$this->urlSettingsArr[$r][$k] = $op;
			}
		}
	}

	private function geturlcron_set_cronjoboptions() {
		for ($r = 1; $r <= self::$nooffields; $r++) {
			foreach(self::$fi as $k => $v) {
				$ki = "geturlcron-".$k."-".$r;
				$op = update_option($ki, $this->geturlcron_handlePost_input($ki));
			}
		}
	}

	public function geturlcron_add_action_cronjob() {
					$cfArr = explode("-", current_filter());
					$no = trim($cfArr[1]); 
					if (!empty($this->urlSettingsArr[$no])) {
						$urltouse = $this->urlSettingsArr[$no];
						$this->geturlcron_executejob($urltouse, $no); 
					}
	}

	public function geturlcron_activatejobs() {
		for ($nol = 1; $nol <= count($this->urlSettingsArr); $nol++) {
			if (!empty($this->urlSettingsArr[$nol])) {
				$urltouse = $this->urlSettingsArr[$nol];
				$this->geturlcron_schedule($urltouse, $nol);
			}
		}
	}

	public function geturlcron_check_mailadress_list($mailadress_list) {
		$mailadr = preg_replace("/[, ]/", ";", $mailadress_list);
		$mailadrArr = preg_split("/[,; ]/", $mailadr);
		$out = "";
		foreach( $mailadrArr as $k => $v) {
			if (filter_var($v, FILTER_VALIDATE_EMAIL)) {
					$out .= "OK: $v<br>";
			} else {
					if (!empty($v)) {
						$out .= "<font color=red>CHECK E-Mailadress please: $v</font><br>";
					}
			}
		}
		return $out;
	}
	
	public function geturlcron_action_handle() {
	
		$this->subaction = $this->geturlcron_handlePost_input("subaction");
		if ("settings"==$this->subaction) {
			$input_geturlcron_timeout = $this->geturlcron_input_integer($this->geturlcron_handlePost_input("geturlcron-timeout"), 60, TRUE, TRUE, TRUE);
			update_option("geturlcron-timeout", $input_geturlcron_timeout);
			
			$input_geturlcron_dellog_days = $this->geturlcron_input_integer($this->geturlcron_handlePost_input("geturlcron-dellog-days"), 20, TRUE, FALSE, TRUE);
			update_option("geturlcron-dellog-days", $input_geturlcron_dellog_days);
			
			$input_geturlcron_emailadr = sanitize_email($this->geturlcron_handlePost_input("geturlcron-emailadr"));
			update_option("geturlcron-emailadr", $input_geturlcron_emailadr);
			
			$input_geturlcron_uninstall_deleteall = $this->geturlcron_handlePost_input("geturlcron-uninstall-deleteall");
			if (1!=$input_geturlcron_uninstall_deleteall) {
				$input_geturlcron_uninstall_deleteall  = 0;
			}
			update_option("geturlcron-uninstall-deleteall", $input_geturlcron_uninstall_deleteall);
		}
		if ("savecronjobs"==$this->subaction) {
			# remove all jobs
			$this->geturlcron_unschedulejobs();

			# create all jobs
			$this->geturlcron_set_cronjoboptions();
			$this->geturlcron_set_urlSettingsarr();
			$this->geturlcron_activatejobs();
		}
		
		$this->action = $this->geturlcron_handleGet_input("action");
		if ("geturlcron"==$this->action) {
			$noncecheckok = wp_verify_nonce($_REQUEST["hash"], "getcronurl"); 
			if (!$noncecheckok) {
				return TRUE;
			}
			$no = sanitize_text_field($_GET["no"]);
			if (is_numeric($no) && $no>0) {
				$this->geturlcron_singlerun($no);
			}
		}
		return TRUE;
	}

	private function geturlcron_input_integer($input, $defaultvalue, $isnumeric, $ispositive, $round) {
		if ($isnumeric && !is_numeric($input)) {
			$input = $defaultvalue;
		}
		if ($ispositive && $input<=0) {
			$input = $defaultvalue;
		}
		if ($round && $input!=round($input)) {
			$input = round($input+0.5);
		}
		return $input;
	}
	
	private function geturlcron_handlePost_input($postparm) {
		$pp = sanitize_text_field(@$_POST[$postparm]);
		return $pp;
	}
	private function geturlcron_handleGet_input($postparm) {
		$pp = sanitize_text_field(@$_GET[$postparm]);
		if ("geturlcron"!=$pp) {
			return "";
		}
		return $pp;
	}


	private function geturlcron_getschedule_interval($urlArr, $no) {
		$autoadd = "";
		$retVal = array();
		$schedurl = trim($urlArr["url"]);
		if (empty($schedurl)) {
			return $retVal;
		}
		$sedeuleofurl = $urlArr["interval"];
		$timefirstexec = strtotime($urlArr["startdate"]);
		if (empty($timefirstexec)) {
			$timefirstexec = current_time( 'timestamp' );
		}
		if (time()>$timefirstexec) {
			# first run in past: set to next
			$schedules = wp_get_schedules();
			$secintv = $schedules[ $sedeuleofurl ][ 'interval' ];  ## interval in seconds
			if ($secintv>0) {
				$numberOfIntervalsTillNextExec = round(((time()-$timefirstexec)/$secintv) + 0.5);
			} else {
				$numberOfIntervalsTillNextExec = 0;
			}
			$nextExecTime = $timefirstexec + $numberOfIntervalsTillNextExec * $secintv;
			
			$autoadd .= "interval in sec: $secintv, ";
			$autoadd .= "firstexec: $timefirstexec (".$urlArr["startdate"]."), ";
			$autoadd .= "numberOfIntervalsTillNextExec: $numberOfIntervalsTillNextExec, ";
			$autoadd .= "nextExecTime: $nextExecTime (".date("Y-m-d, H:i:s", $nextExecTime)."), ";
		} else {
			# first run in the future
			$nextExecTime = $timefirstexec;
		}
		
		$retVal{"sedeuleofurl"} = $sedeuleofurl;
		$retVal{"autoadd"} = $autoadd;
		$retVal{"timenextexec"} = $nextExecTime;
		
		return $retVal;
		
	}

	private function geturlcron_singlerun($no) {
		$logl = $this->geturlcron_log("", $this->urlSettingsArr[$no]["url"], "", "", "", "", "manually started");
		$this->geturlcron_savelog($logl);
		wp_schedule_single_event( time(), 'geturlcron_event-'.$no);
	}	


	private function geturlcron_schedule($urlArr, $no) {
		$retVal = $this->geturlcron_getschedule_interval($urlArr, $no);
		if (!empty($retVal{"sedeuleofurl"})) {
			$logl = $this->geturlcron_log("geturlcron-$no", $urlArr["url"], "", "interval: ".$retVal{"sedeuleofurl"}.", ".$retVal{"autoadd"}." next run: ".date("Y-m-d, H:i:s", $retVal{"timenextexec"}), "", "", "schedule");
			$this->geturlcron_savelog($logl);
			if ($retVal{"timenextexec"}>0) {
				wp_schedule_event( $retVal{"timenextexec"}, $retVal{"sedeuleofurl"}, 'geturlcron_event-'.$no);
			}
		}
	}	

	private function geturlcron_log($idofrun, $url, $done_retries, $returnvalue, $info, $runtime, $status) {
		$separator = "#gucsep#";
		$separator = " /// ";
		$separator2 = "=";
		$logline = $idofrun.$separator."time".$separator2.time().$separator."date".$separator2.date("Y-m-d, H:i:s").$separator.
				"retries".$separator2.$done_retries.$separator.$info.$separator."url".$separator2.$url.$separator."runtime".$separator2.$runtime.$separator."json".$separator2.substr($returnvalue, 0 ,300).$separator.$status;
		$logline = preg_replace("/\n/", "", $logline);
		$logline = preg_replace("/\r/", "", $logline);
		return $logline;
	}


	private static function geturlcron_setlogfile() {
		#$plugincachepath = plugin_dir_path(__FILE__) . "logs";
		$ulp = wp_upload_dir();
		$plugincachepath = $ulp["basedir"]."/geturlcron";
		if (!is_dir($plugincachepath)) {
          $mkdirError = @mkdir($plugincachepath, 0777 , TRUE);
		}
		self::$logfile = $plugincachepath."/geturlcron-log.cgi";
	}
	public static function geturlcron_getlogfile() {
		return self::$logfile;
	}
	public static function geturlcron_getnooffields() {
		return self::$nooffields;
	}


	private function geturlcron_savelog($logline) {
		$logfile = $this->geturlcron_getlogfile();
		$fsc = file_put_contents($logfile, $logline."\n", FILE_APPEND);
	}

	private function geturlcron_getandcheckurl($url, $done_retries, $idofrun, $no) {
		$urlout = trim(stripslashes($url));
		$this->getcronurl_sendmail("TRY $done_retries: get $url, ID $idofrun", "Try $done_retries:\nget $urlout\nID $idofrun", $no);
		$logl = $this->geturlcron_log($idofrun, $urlout, $done_retries, "start trying", "", "", "try");
		$this->geturlcron_savelog($logl);

		$timeout = trim(get_option('geturlcron-timeout'));
		if (!($timeout>0)) {
			$timeout = "60";
		}
		$starttime = time();

		if (preg_match("/^\[/",$url)) {
			$sc = trim(stripslashes($url));
			$returnvalue = do_shortcode($sc);
			$resp = "shortcode"; # do_shortcode does not have an errorlevel
		} else {
			$args = array(
				'timeout'     	=> $timeout,
				'user-agent'	=> 'GetURLCron-Plugin',
				'sslverify' 	=> false
				);
			$response = wp_remote_get($url, $args);
			if ( is_wp_error( $response ) ) {
				$error_message = $response->get_error_message();
				return $error_message;
			}
			$resp = wp_remote_retrieve_response_code($response); # http code
			$returnvalue = wp_remote_retrieve_body($response);
		}
		
		$endtime = time();
		$runtime = $endtime - $starttime;
		return array(
			"starttime" => $starttime,
			"endtime" => $endtime,
			"runtime" => $runtime,
			"returnvalue" => $returnvalue,
			"resp" => $resp,
		);
	}

	public function geturlcron_executejob($urlArr, $no) {
		$url = $urlArr["url"];
		if (empty($url)) {
			return TRUE;
		}
		$urlout = stripslashes($url);
				
		$retries = $urlArr["retries"];
		$overallok = FALSE;
		$done_retries = 1;

		$idofrun = md5(time().$url.rand());

		####
		$retArr = $this->geturlcron_getandcheckurl($url, $done_retries, $idofrun, $no);
		$returnvalue = $retArr["returnvalue"];
		$resp = $retArr["resp"];
		$runtime = $retArr["runtime"];
		$starttime = date("H:i:s", $retArr["starttime"]);
		$endtime = date("H:i:s", $retArr["endtime"]);

		#$message = " failedcheck \n";

		$checkArray = $this->geturlcron_checkresponse($urlArr, $returnvalue, $resp);
		if ($checkArray["requestok"]) {
		#if ($checkArray[""]) {
			$overallok = TRUE;
		} else { 
			#$message .= " fail detected \n";
			if ($retries>1) { # try again
				#$message .= " retry due to fail: done_retry is $done_retries \n";
				for ($r = 1; $r <= $retries; $r++) {
					#$message .= " retry run $r \n";
					$checkArray = $this->geturlcron_checkresponse($urlArr, $returnvalue, $resp);
					$done_retries++;
					$retArr = $this->geturlcron_getandcheckurl($url, $done_retries, $idofrun, $no);
					if ($checkArray["requestok"]) { # try ok
						#$message .= " retry run $r OK \n";
						$returnvalue = $retArr["returnvalue"];
						$resp = $retArr["resp"];
						$runtime = $retArr["runtime"];
						$starttime = date("H:i:s", $retArr["starttime"]);
						$endtime = date("H:i:s", $retArr["endtime"]);
						$overallok = TRUE;
						break;
					} else {
						#$message .= " retry run $r failed \n";
					}
				}
			} else {
				# no retry
			}
		}
	
		if ($overallok) {
			$status = "OK ";
		} else {
			$status = "FAIL";
		}
	
		$info = $status." ".$checkArray["info"];
		$cronname = "$no, retires: $done_retries, url: $urlout, result: ".$logl;

		$logl = $this->geturlcron_log($idofrun, $urlout, $done_retries, $returnvalue, $info, $runtime, $status);
		$this->geturlcron_savelog($logl);

		$subject = $status." $done_retries: get $urlout, ID $idofrun";

		$message = "\n--- Status:\n";
		$message .= "Status: $status\n\n";
		$message .= "Job: $no\nURL: $urlout\n";
		$message .= "ID: $idofrun\n";
		$message .= "retires: $done_retries\n";
		$info4mail = $checkArray["info4mail"];
		$message .= "$info4mail\n";
		$message .= "\n--- Runtime:\n";
		$message .= "time for URL-get: \n".$runtime." seconds\n";
		$message .= "timewindow:\n$starttime to $endtime\n";
		$this->getcronurl_sendmail($subject, $message, $no);
	}

	private function getcronurl_sendmail($subject, $message, $no) {
		$doSendFlag = $this->urlSettingsArr[$no]["sendmail"];
		if ("yes"==$doSendFlag) {
			$to = trim(get_option('geturlcron-emailadr'));
			$to = preg_replace("/[, ]/", ";", $to);
			if (!empty($to)) {
				$resmail = mail( $to , $subject , $message);
			}
		} else {
			/*
			$to = trim(get_option('geturlcron-emailadr'));
			$to = preg_replace("/[, ]/", ";", $to);
			if (!empty($to)) {
				$resmail = mail( $to , "do not send" , $message);
			}
			*/
		}
	}

	private function geturlcron_checkOnJSONfield($jsonArr, $fieldkeypath) {
		$jsonfieldfound = FALSE;
		$foundvalue = "JSON-Fieldcheck in detail:\n";

		$reqFieldArr = explode(".", $fieldkeypath);
		$jsonArrTmp = $jsonArr;
		
		foreach( $reqFieldArr as $reqFieldDir) {
			if ($reqFieldDir=="#LASTITEM#") {
				# data.#LASTITEM#.no
				$reqFieldDir = count($jsonArrTmp)-1;
			}
			$jsonArrTmp = $jsonArrTmp[trim($reqFieldDir)];
			$foundvalue .= "check on $reqFieldDir : ".count($jsonArrTmp)." found items\n";
		}
		if (!empty($jsonArrTmp)) {
			$jsonfieldfound = TRUE;
			$foundvalue .= "value found: ".print_R($jsonArrTmp, true)."\n";
		} else {
			$foundvalue .= "NO value found: check failed"."\n";
		}
		
		return array(
			"jsonfieldfound" => $jsonfieldfound,
			"foundvalue" => $foundvalue,
		);
	}

	private function geturlcron_checkresponse($urlArr, $returnvalue, $resp) {
		$reqok = FALSE;
		$jsonok = FALSE;
		$requestok = FALSE;
		$info = "";
		$info4mail = "\n--- Request:\n";
		if (
			(200==$resp) ||  
			("shortcode"==$resp) 
			){
			# server answer: ok
			$reqok = TRUE;
			$info .= "request ok-";
			$info4mail .= "request ok: $resp\n";
		} else {
			$info .= "request: $resp";
			$info4mail .= "request failed: $resp\n";
		}
		if ($reqok) {
			$requiredformat = $urlArr["requiredformat"];
			if ($requiredformat=="json") { # check on json
				$info4mail .= "check on valid json\n";
				# json ok?
				$jsonArr = json_decode($returnvalue, TRUE);
				if (is_null($jsonArr)) {
					$info .= "json invalid-";
					$info4mail .= "json is invalid\n";
				} else {
					$info .= "json valid-";
					$info4mail .= "json is valid\n";
					$jsonok = TRUE;
					$requiredjsonfield = trim($urlArr["requiredjsonfield"]);
					if (empty($requiredjsonfield)) { 
						$requestok = TRUE;
						$info .= "no check for required jsonfield-";
						$info4mail .= "no check for required jsonfield\n";
					} else {
						$checkOnJSONArr = $this->geturlcron_checkOnJSONfield($jsonArr, $requiredjsonfield);
						if ($checkOnJSONArr["jsonfieldfound"]) { 
						#if (empty($jsonArr[$requiredjsonfield])) {
							$info .= "required jsonfield ok-";
							$info4mail .= "required jsonfield ok: $requiredjsonfield\n";
							$info4mail .= "found jsonvalue: ".$checkOnJSONArr["foundvalue"]."\n";
							$requestok = TRUE;
						} else {
							$info .= "required jsonfield missing: $requiredjsonfield-";
							$info4mail .= "required jsonfield missing: $requiredjsonfield\n";
							$info4mail .= "failed jsonvalue detection:\n".$checkOnJSONArr["foundvalue"]."\n";
						}
					}
				}
			} else {
				# any format welcome, check on requiredfield
				$requiredjsonfield = trim($urlArr["requiredjsonfield"]);
				$info4mail .= "check on string: requiredjsonfield\n";
				if (empty($requiredjsonfield)) { 
					$requestok = TRUE;
					$info .= "no check for required string-";
					$info4mail .= "no check for required string\n";
				} else {
					if (preg_match("/$requiredjsonfield/", $returnvalue)) {
						$info .= "required string ok-";
						$info4mail .= "required string ok: $requiredjsonfield\n";
						$requestok = TRUE;
					} else {
						$info .= "required string missing: $requiredjsonfield-";
						$info4mail .= "required string missing: $requiredjsonfield\n";
					}
				}
			}
		}
		return array(
			"info" => $info,
			"info4mail" => $info4mail,
			"reqok" => $reqok,
			"jsonok" => $jsonok,
			"requestok" => $requestok,
		);
}

	public function geturlcron_unschedulejobs() {
		for ($no = 1; $no <= count($this->urlSettingsArr); $no++) {
			$this->geturlcron_unschedulejob($no);
		}
	}
	
	public function geturlcron_unschedulejob($no) {
		$jobhook = "geturlcron_event-".$no;
		wp_unschedule_event( "", $jobhook );
		wp_clear_scheduled_hook( $jobhook );
	}


	## scheduling intervals
	public function geturlcron_recurrence_interval( $schedules ) {
		$schedules['geturlcron_02_minutes'] = array(
            'interval'  => 60*2,
            'display'   => '2 Minutes'
		);
		$schedules['geturlcron_30_minutes'] = array(
            'interval'  => 60*30,
            'display'   => '30 Minutes'
		);
		$schedules['geturlcron_6_hours'] = array(
            'interval'  => 60*60*6,
            'display'   => '6 Hours'
		);
		$schedules['geturlcron_7_days'] = array(
            'interval'  => 60*60*24*7,
            'display'   => '7 Days'
		);
		return $schedules;
	}


	public static function initclass() {
		static $inst = null;
		if ( ! $inst ) {
			$inst = new GetUrlCron();
		}
		return $inst;

	}
}
GetUrlCron::initclass();
?>